#include "input.h"
#include "ui.h"
#include "uielements.h"
#include "internal.h"

namespace kek::Defaults {

static KeyBinding
	keyForward,
	keyBackward,
	keyLeft,
	keyRight,
	keyUp,
	keyDown,
	keyOptions,
	keyToggleCursorMode;

static ButtonElement *options;

static void defaultInput(GLFWwindow *window, void *data) {
	if(Input::getKeyState(keyForward) == GLFW_PRESS) {
		kekData.activeCamera->translate(kekData.activeCamera->direction * KEK_NOCLIP_SPEED * kekData.lastFrameTime);
	}

	if(Input::getKeyState(keyBackward) == GLFW_PRESS) {
		kekData.activeCamera->translate(kekData.activeCamera->direction * -KEK_NOCLIP_SPEED * kekData.lastFrameTime);
	}

	if(Input::getKeyState(keyLeft) == GLFW_PRESS) {
		glm::vec3 camRight = glm::normalize(glm::cross(kekData.activeCamera->direction, glm::vec3(0.0f, 1.0f, 0.0f)));
		kekData.activeCamera->translate(-camRight * KEK_NOCLIP_SPEED * kekData.lastFrameTime);
	}

	if(Input::getKeyState(keyRight) == GLFW_PRESS) {
		glm::vec3 camRight = glm::normalize(glm::cross(kekData.activeCamera->direction, glm::vec3(0.0f, 1.0f, 0.0f)));
		kekData.activeCamera->translate(camRight * KEK_NOCLIP_SPEED * kekData.lastFrameTime);
	}

	if(Input::getKeyState(keyUp) == GLFW_PRESS) {
		kekData.activeCamera->translateY(KEK_NOCLIP_SPEED * kekData.lastFrameTime);
	}

	if(Input::getKeyState(keyDown) == GLFW_PRESS) {
		kekData.activeCamera->translateY(-KEK_NOCLIP_SPEED * kekData.lastFrameTime);
	}
}

static void defaultKeyCallback(GLFWwindow *window, int key, int scancode, int action, int mods, void *data) {
	if(key == Input::getKeyBinding(keyOptions).key && action == GLFW_PRESS) {
		options->visible = !options->visible;
	}

	if(key == Input::getKeyBinding(keyToggleCursorMode).key && action == GLFW_PRESS) {
		if(Input::getCursorMode() == GLFWCursorMode::CAPTURE) {
			Input::setCursorMode(GLFWCursorMode::FREE);
		}else {
			Input::setCursorMode(GLFWCursorMode::CAPTURE);
		}
	}
}

static void defaultMouseCallback(GLFWwindow *window, double x, double y, void *data) {
	switch(Input::getCursorMode()) {
		case GLFWCursorMode::CAPTURE:
		{
			static bool firstMouse = true;
			static double lastX = 0, lastY = 0;
			if(firstMouse) {
				lastX = x;
				lastY = y;
				firstMouse = false;
			}

			float xoff = lastX - x;
			float yoff = lastY - y;
			lastX = x;
			lastY = y;

			xoff *= 0.1f;
			yoff *= 0.1f;

			kekData.activeCamera->rotateYaw(xoff);
			kekData.activeCamera->rotatePitch(yoff);
		}
		case GLFWCursorMode::FREE:
		case GLFWCursorMode::HIDDEN:
		{
			for(UIElement *element : kekData.uiElements) {
				UIPoint childPos = element->getPosition();
				if(element->hoverAll(UIPoint((int) x - childPos.x, (int) y - childPos.y), UIPoint((int) x, (int) y))) break;
			}
		}
	}
}

static void defaultMouseButtonCallback(GLFWwindow* window, int button, int action, int mods, void *data) {
	switch(Input::getCursorMode()) {
		case GLFWCursorMode::CAPTURE:
		{
			// TODO
		}
		case GLFWCursorMode::FREE:
		case GLFWCursorMode::HIDDEN:
		{
			switch(action) {
				case GLFW_PRESS:
					break;
				case GLFW_RELEASE:
				{
					double x, y;
					glfwGetCursorPos(window, &x, &y);
					for(UIElement *element : kekData.uiElements) {
						UIPoint childPos = element->getPosition();
						if(element->clickAll(UIPoint((int) x - childPos.x, (int) y - childPos.y), UIPoint((int) x, (int) y), (GLFWMouseButton) button)) break;
					}
				}
			}
		}
	}
}

void init() {
	keyForward = Input::createKeyBinding("Forward", GLFW_KEY_W);
	keyBackward = Input::createKeyBinding("Backward", GLFW_KEY_S);
	keyLeft = Input::createKeyBinding("Left", GLFW_KEY_A);
	keyRight = Input::createKeyBinding("Right", GLFW_KEY_D);
	keyUp = Input::createKeyBinding("Up", GLFW_KEY_SPACE);
	keyDown = Input::createKeyBinding("Down", GLFW_KEY_LEFT_CONTROL);
	keyOptions = Input::createKeyBinding("Options", GLFW_KEY_Q);
	keyToggleCursorMode = Input::createKeyBinding("Toggle Cursor Mode", GLFW_KEY_TAB);

	Input::addPeriodicCallback(PeriodicCallback(defaultInput, nullptr));
	Input::addKeyListener(KeyCallback(defaultKeyCallback, nullptr));
	Input::addMouseListener(MouseCallback(defaultMouseCallback, nullptr));
	Input::addMouseButtonListener(MouseButtonCallback(defaultMouseButtonCallback, nullptr));

	options = new ButtonElement(px(0), px(100), px(100), px(50));
	//UI::addElement(options);
}

}
