#include "input.h"

#include <map>
#include <iostream>

#include "internal.h"

namespace kek::Input {

static InputListener nextID = 0;

InputListener addPeriodicCallback(PeriodicCallback callback) {
	InputListener id = nextID++;
	kekData.periodicCallbacks.emplace(id, callback);
	return id;
}

void removePeriodicCallback(InputListener listener) {
	kekData.periodicCallbacks.erase(listener);
}

InputListener addKeyListener(KeyCallback callback) {
	InputListener id = nextID++;
	kekData.keyCallbacks.emplace(id, callback);
	return id;
}

void removeKeyListener(InputListener listener) {
	kekData.keyCallbacks.erase(listener);
}

InputListener addMouseListener(MouseCallback callback) {
	InputListener id = nextID++;
	kekData.mouseCallbacks.emplace(id, callback);
	return id;
}

void removeMouseListener(InputListener listener) {
	kekData.mouseCallbacks.erase(listener);
}

InputListener addMouseButtonListener(MouseButtonCallback callback) {
	InputListener id = nextID++;
	kekData.mouseButtonCallbacks.emplace(id, callback);
	return id;
}

void removeMouseButtonListener(InputListener listener) {
	kekData.mouseButtonCallbacks.erase(listener);
}

KeyBinding createKeyBinding(std::string name, GLFWKey defaultKey) {
	if(name == KEK_INVALID_KEY_BINDING_NAME) return KEK_INVALID_KEY_BINDING;
	KeyBinding id = nextID++;
	KeyBindingData d;
	d.name = name;
	d.key = defaultKey;
	kekData.keyBindings.emplace(id, d);
	return id;
}

void reassignKeyBinding(KeyBinding binding, GLFWKey key) {
	if(binding == KEK_INVALID_KEY_BINDING) return;
	auto it = kekData.keyBindings.find(binding);
	if(it == kekData.keyBindings.end()) return;
	KeyBindingData d = it->second;
	d.key = key;
	it->second = d;
}

KeyBindingData getKeyBinding(KeyBinding binding) {
	auto it = kekData.keyBindings.find(binding);
	if(it == kekData.keyBindings.end()) return {.name = KEK_INVALID_KEY_BINDING_NAME};
	return it->second;
}

GLFWKeyState getKeyState(KeyBinding binding) {
	auto it = kekData.keyBindings.find(binding);
	if(it == kekData.keyBindings.end()) return -1;
	return glfwGetKey(kekData.window, it->second.key);
}

void setCursorMode(GLFWCursorMode mode) {
	glfwSetInputMode(kekData.window, GLFW_CURSOR, (int) mode);
	kekData.uiCursorMode = mode;
}

GLFWCursorMode getCursorMode() {
	return kekData.uiCursorMode;
}

}
